


AJS.toInit(function() {
	var baseUrl = AJS.$("meta[name='application-base-url']").attr("content");

	var nextFieldIndex = 0;

	// Stores the IdP attributes read from the metadata
	var idpAttributeMaps = {};

	// Setting the baseUrl meta tag in confluence is not working, but there is a provided
	// meta-tag called confluence-base-url that does the job.
	if(typeof baseUrl === "undefined") {
		baseUrl = AJS.$("meta[name='confluence-base-url']").attr("content");
	}

	function disableForm() {
		AJS.$(".disableable :input").prop('disabled',true);
		AJS.$("#waitmessage").show();
	}

	function enableForm() {
		AJS.$(".disableable :input").prop('disabled',false);
		AJS.$("#waitmessage").hide();
		window.scrollTo(0, 0);
	}

	function populateForm() {
		console.log("Loading configuration data");
		disableForm();
		AJS.$.ajax({
			url: baseUrl + "/rest/samlsso-admin/1.0/",
			dataType: "json",
			success: function(config) {
				console.log("Filling form fields");
				fillFields(config);

				AJS.$("#licenseMessage").text(config.licenseMessage);
				if(config.licensed) {
					AJS.$("#licenseMessage").removeClass("error");
					AJS.$("#licenseMessage").addClass("success");
				} else {
					AJS.$("#licenseMessage").addClass("error");
					AJS.$("#licenseMessage").removeClass("success");    	
				}
				enableForm();
			},
			error: function() {
				console.log("Loading configuration data failed");
				AJS.$("#aui-message-bar").empty(); 
				AJS.messages.error({
					title: "Load failed",
					body: "Loading settings failed!",
					closeable: false
				});
				enableForm();
			}
		});
	}

	/*
	 * Writes the config into the form elements. 
	 * The value is assigned to the form element having the key as name attribute.
	 */
	function fillFields(config) {
		AJS.$.each(config, function(k,v) {
			if(k == 'idpConfigurations') {
				addIdPConfigurationFields(v);
			} else {
				var formElement = AJS.$("*[name=" + k + "]");
				if (formElement.length == 0) {
					console.log("Found no element with name " + k);
				} else {
					if(formElement.attr("type") == "checkbox") {
						formElement.prop('checked',v);
					} else  {
						if(formElement.hasClass('newlineseparated')) {
							v = replaceCommasWithNewlines(v);
						}	
						formElement.val(v);
					}
				} 
			}
		});
	}

	/**
	 * Creates the form fields for the IdP configuration from the SOY-template on admin_tpl.soy
	 */
	function createIdpFields(index, parentElement) {
		//Call the soy-created template and append it to the container element
		var container = AJS.$(Resolution.SAMLSSO.idpsettings({index: index}));
		parentElement.append(container);	
		// Add some Javascript-stuff
		AJS.$("#deletebutton-idp"+index).click(function(e){ container.remove() });

		AJS.$("#metadataerror-idp"+index).hide();
		AJS.$("#certificate-idp"+index).on("change paste",function(e) { checkCertificate(AJS.$("#certificate-idp"+index), AJS.$("#certstatus-idp"+index), AJS.$("#decoded-cert-idp"+index)) });

		addMetadataFields(index);
		
		AJS.$('#metadata-dialog-show-button-idp'+index).click(function(e) { AJS.dialog2('#metadata-dialog-idp'+index).show(); });
		
		AJS.$('#radioButtonMetadataFromURL'+index).change(function(e) {addMetadataFields(index)});
		AJS.$('#radioButtonMetadataFromXML'+index).change(function(e) {addMetadataFields(index)});
		
		//Update the selectors if the field values are modified
	    AJS.$("#useridAttribute-idp"+index).change( function(e) { 
	    	AJS.$("#selector-useridAttribute-idp"+index).val(AJS.$("#useridAttribute-idp"+index).val()) 
	    });
	    
	    AJS.$("#fullnameAttribute-idp"+index).change( function(e) { 
	    	AJS.$("#selector-fullnameAttribute-idp"+index).val(AJS.$("#fullnameAttribute-idp"+index).val()) 
	    });
	    
	    AJS.$("#emailAttribute-idp"+index).change( function(e) { 
	    	AJS.$("#selector-emailAttribute-idp"+index).val(AJS.$("#emailAttribute-idp"+index).val()) 
	    });
	    
	    AJS.$("#groupAttribute-idp"+index).change( function(e) { 
	    	AJS.$("#selector-groupAttribute-idp"+index).val(AJS.$("#groupAttribute-idp"+index).val()) 
	    });
	
	}

	/**
	 * Populates the IdP confiugration fields with data from the config
	 */
	function fillIdPConfigurationFields(index,config) {
		var idpid = config["id"];
		AJS.$("#id-idp"+index).text(idpid);

		idpAttributeMaps[index] = config.idpAttributes;
		// Add the options to the userid-dropdown
		updateIdPSelectFieldOptions(index);

		for(var key in config) {
			var value = config[key];
			var elementId = key + "-idp" + index;
			var formElement = AJS.$("*[id="+ elementId + "]");
			if (formElement.length == 0) {
				console.log("Found no element with id " + elementId);
			} else {
				if(formElement.attr("type") == "checkbox") {
					formElement.prop('checked',value);
				} else  {
					if(formElement.hasClass('newlineseparated')) {
						value = replaceCommasWithNewlines(value);
					}
					formElement.val(value);	
					formElement.trigger("changed");
				}
			} 		
		}	
		checkCertificate(AJS.$("#certificate-idp"+index), AJS.$("#certstatus-idp"+index), AJS.$("#decoded-cert-idp"+index))
		
		//Set the selector according to the field
		AJS.$("#selector-useridAttribute-idp"+index).val(AJS.$("#useridAttribute-idp"+index).val());
		AJS.$("#selector-fullnameAttribute-idp"+index).val(AJS.$("#fullnameAttribute-idp"+index).val());
		AJS.$("#selector-emailAttribute-idp"+index).val(AJS.$("#emailAttribute-idp"+index).val());
		AJS.$("#selector-groupAttribute-idp"+index).val(AJS.$("#groupAttribute-idp"+index).val());
		
	}

	function addIdPConfigurationFields(idpConfigurations) {	
		var container = AJS.$("#idpConfigurationDiv");
		container.empty();	
		if(idpConfigurations.length == 0) {
			console.log("No IdPs in configuration, creating an empty field set");
			nextFieldIndex = 1;
			addNewIdPFields();
		} else {
			for(var i = 0, size = idpConfigurations.length; i<size;i++) {
				var current = idpConfigurations[i];
				createIdpFields(i,container);
				fillIdPConfigurationFields(i,current);
			}
			nextFieldIndex = idpConfigurations.length;
		}
	}

	function addNewIdPFields() {
		nextFieldIndex++;
		var container = AJS.$("#idpConfigurationDiv");
		createIdpFields(nextFieldIndex,container);
	}

	/**
	 * Creates the Form Fields for Metadata URL and Metadata XML
	 */
	function addMetadataFields(index) {
		if(AJS.$('#radioButtonMetadataFromURL'+index).is(':checked')) {
			AJS.$('#metadatadiv-idp'+index).empty();
			AJS.$('#metadatadiv-idp'+index).append(Resolution.SAMLSSO.metadataUrl({index: index}));		
		}
		if(AJS.$('#radioButtonMetadataFromXML'+index).is(':checked')) {
			AJS.$('#metadatadiv-idp'+index).empty();
			AJS.$('#metadatadiv-idp'+index).append(Resolution.SAMLSSO.metadataXml({index: index}));		
		}
		
		AJS.$('#dialog-close-button-idp'+index).click(function(e){ AJS.dialog2('#metadata-dialog-idp'+index).hide();});
		AJS.$('#loadbutton-idp'+index).click(function(e){ readMetadata(index); });
		AJS.$("#metadataerror-idp"+index).empty();
		AJS.$("#metadataerror-idp"+index).hide();
	}

	function readIdPFields() {
		var idpData = [];
		AJS.$("#idpConfigurationDiv").children().each( function(index,element){
			var data = {}			
			var idpId = AJS.$(element).find("span[id*='id-idp']").first().text();
			if(idpId == "NEW") {
				idpId = -1;
			} 
			data['id'] = idpId;

			AJS.$(element).find("input:not([type='submit'],[type='reset']),textarea,select").each( function(index,formElement) {
				var formId = AJS.$(formElement).attr("id");
				//console.log(formId);
				if (formId === undefined) {
					console.log(formElement);
				} else {
					var key = formId.replace(/-idp.*$/,'');
					var value;
					if(AJS.$(formElement).attr("type") == "checkbox") {
						value = AJS.$(formElement).prop('checked');	// Checkboxes need to be checked for the checked attribute
					} else {
						value = AJS.$(formElement).val(); // from all other elements the value can be retrieved with the val() function 
						if(AJS.$(formElement).hasClass('newlineseparated')) {
							value = replaceNewlinesWithCommas(value);
						}
					}
					data[key] = value;
				}
			});
			console.log(idpAttributeMaps);
			data['idpAttributes'] = idpAttributeMaps[index];
			idpData.push(data);
		});
		return idpData;
	}

	/**
	 * Reads the metadata from the URL or XML, sends it to the backend for parsing and fills the form fields
	 * with data from the result map
	 */
	function readMetadata(index) {	
		var url = AJS.$('#metadataurl-idp' + index).val();
		var xml = AJS.$('#metadataxml-idp' + index).val();
		var ignorecert = AJS.$('#ignorecert-idp' + index).prop('checked');

		var fetchingInfo = { 'metadataXML':xml,
				'metadataURL':url,
				'ignoreCertificate':ignorecert
		};

		if((url != undefined && url != "" && url.substr(0,4) == "http") || (xml != undefined && xml != "")) {

			AJS.$("#loadbutton-idp"+index).spin();
			AJS.$("#metadataerror-idp"+index).empty();
			AJS.$("#metadataerror-idp"+index).hide();

			AJS.$.ajax({
				url: baseUrl + "/rest/samlsso-admin/1.0/readMetadata",
				type: "POST",
				contentType: "application/json; charset=utf-8",
				data: JSON.stringify(fetchingInfo),
				processData: true,
				success: function (result) { 
					AJS.$("#issuer-idp"+index).val(result.entityId);
					AJS.$("#url-idp"+index).val(result.ssoPOSTUrl);
					AJS.$("#certificate-idp"+index).val('---BEGIN CERTIFICATE---\n' + result.signingCertificate);
					AJS.$("#certificate-idp"+index).trigger('change');
					AJS.$("#loadbutton-idp"+index).spinStop();

					// Store the values to store them in the configuration
					idpAttributeMaps[index] = result.attributes;

					// Add the options to the userid-dropdown
					updateIdPSelectFieldOptions(index);
					
					// Close the dialog
					AJS.dialog2('#metadata-dialog-idp'+index).hide();
				},
				error: function(result) { 
					AJS.$("#metadataerror-idp"+index).append("<p>" + result.responseText + "</p>");
					AJS.$("#metadataerror-idp"+index).show();
					AJS.$("#loadbutton-idp"+index).spinStop();
				}
			}); 
		}	
	}

	/** 
	 * Updates the options in the select fields
	 */
	function updateIdPSelectFieldOptions(index) {	
	
		setSelectOptions(AJS.$("#selector-useridAttribute-idp"+index),idpAttributeMaps[index]);
		AJS.$("#selector-useridAttribute-idp"+index).unbind("change");
		AJS.$("#selector-useridAttribute-idp"+index).change(function() {
			AJS.$("#useridAttribute-idp"+index).val(AJS.$("#selector-useridAttribute-idp"+index).val());
		});
		
		setSelectOptions(AJS.$("#selector-fullnameAttribute-idp"+index),idpAttributeMaps[index]);
		AJS.$("#selector-fullnameAttribute-idp"+index).unbind("change");
		AJS.$("#selector-fullnameAttribute-idp"+index).change(function() {
			AJS.$("#fullnameAttribute-idp"+index).val(AJS.$("#selector-fullnameAttribute-idp"+index).val());
		});
		
		setSelectOptions(AJS.$("#selector-emailAttribute-idp"+index),idpAttributeMaps[index]);
		AJS.$("#selector-emailAttribute-idp"+index).unbind("change");
		AJS.$("#selector-emailAttribute-idp"+index).change(function() {
			AJS.$("#emailAttribute-idp"+index).val(AJS.$("#selector-emailAttribute-idp"+index).val());
		});
		
		setSelectOptions(AJS.$("#selector-groupAttribute-idp"+index),idpAttributeMaps[index]);
		AJS.$("#selector-groupAttribute-idp"+index).unbind("change");
		AJS.$("#selector-groupAttribute-idp"+index).change(function() {
			AJS.$("#groupAttribute-idp"+index).val(AJS.$("#selector-groupAttribute-idp"+index).val());
		});
	}

	/**
	 * Adds the options to a select field
	 * options is a map, the key is the option's display name, the value the option value
	 */
	function setSelectOptions(selectElement,options) {
		var existingvalue = selectElement.val();
		selectElement.empty();
		selectElement.append('<option value="">other</option>')
		for(var key in options) {
			if (options.hasOwnProperty(key)){	 
				var val = options[key]; 
				selectElement.append('<option value="'+ val + '">' + key + '</option>')
			}
		}
		selectElement.val(existingvalue);
	}

	/**
	 * Reads the data from the form field into a JSON object.
	 * The form element's name attribute is used as key.
	 */
	function readFields() {
		var returnData = {};
		// Iterate through all input elements and textareas expect reset and submit buttons
		returnData["idpConfigurations"] = readIdPFields();

		AJS.$("#readTheseFields").find("input:not([type='submit'],[type='reset']),textarea").each( function(index,formElement) {
			var name = AJS.$(formElement).attr("name");
			var value;
			if(AJS.$(formElement).attr("type") == "checkbox") {
				value = AJS.$(formElement).prop('checked');	// Checkboxes need to be checked for the checked attribute
			} else {
				value = AJS.$(formElement).val(); // from all other elements the value can be retrieved with the val() function 
				if(AJS.$(formElement).hasClass('newlineseparated')) {
					value = replaceNewlinesWithCommas(value);
				}
			}
			returnData[name] = value;	
		});
		return returnData;	
	}

	/**
	 * Gathers the configuration data from the form fields and sends them to the backend
	 */
	function updateConfig() {

		disableForm();
		var ajaxdata = readFields();

		AJS.$.ajax({
			url: baseUrl + "/rest/samlsso-admin/1.0/",
			type: "PUT",
			contentType: "application/json; charset=utf-8",
			data: JSON.stringify(ajaxdata),
			processData: true,
			success: function () { 
				AJS.$("#aui-message-bar").empty(); 
				AJS.messages.success({
					title: "Saved",
					body: "Settings successfully saved.",
					closeable: false
				});
				populateForm();
			},
			error: function(result) { 
				var errorMessages = AJS.$.parseJSON(result.responseText);

				var errorList = AJS.$('<ul>',{"class":"validationMessages"});
				for(i=0; i<errorMessages.length; i++) {
					console.log(errorMessages[i]);
					errorList.append(AJS.$("<li>",{text:errorMessages[i]}));
				}
				var msgBody = AJS.$("<div>");
				msgBody.append(errorList);

				AJS.$("#aui-message-bar").empty(); 
				AJS.messages.error({
					title: "Save failed",
					body: msgBody.html(),
					closeable: false
				}); 
				enableForm();
			}
		});  
	}

	/**
	 * Checks the BASE64-encoded certificate by sending the data to the backend
	 */
	function checkCertificate(certField, lozenge, readableDiv) {
		var certString = certField.val().trim();

		if(certString == "") {
			AJS.$(lozenge).removeClass();
			AJS.$(lozenge).addClass("aui-lozenge aui-lozenge-current");
			AJS.$(lozenge).text("not set");
			AJS.$(readableDiv).text("No certificate set");
		} else {
			var spinner = AJS.$("<span>",{"class":"aui-icon aui-icon-wait"});
			AJS.$(lozenge).removeClass();
			AJS.$(lozenge).addClass("aui-lozenge");
			AJS.$(lozenge).text("checking ");
			AJS.$(lozenge).append(spinner);

			AJS.$.ajax({
				url: baseUrl + "/rest/samlsso-admin/1.0/checkCertificate",
				type: "PUT",
				contentType: "text/plain; charset=utf-8",
				data: certString,
				success: function(result) { 
					AJS.$(lozenge).empty();
					AJS.$(lozenge).removeClass();
					AJS.$(lozenge).addClass("aui-lozenge aui-lozenge-success");
					AJS.$(lozenge).text("OK");
					AJS.$(readableDiv).html(escapeHtmlAndNewlines(result));

				},
				error: function(result) { 
					AJS.$(lozenge).empty();
					AJS.$(lozenge).removeClass();
					AJS.$(lozenge).addClass("aui-lozenge aui-lozenge-error");
					AJS.$(lozenge).text("Validity check failed");
					AJS.$(readableDiv).text(result.responseText);
				},
				statusCode: {
					400: function(result) {
						AJS.$(lozenge).empty();
						AJS.$(lozenge).removeClass();
						AJS.$(lozenge).addClass("aui-lozenge aui-lozenge-error");
						AJS.$(lozenge).text("Invalid");
						AJS.$(readableDiv).text(result.responseText);	
					}
				}		
			});  
		}
	}

	populateForm();

	AJS.$("#admin").submit(function(e) {
		e.preventDefault();
	});

	AJS.$("#saveButton").click(function(e) {
		updateConfig();
	});

	AJS.$("#cancelButton").click(function(e) {
		AJS.$("#aui-message-bar").empty(); 
		populateForm();
	});

	AJS.$("#addIdPButton").click(function(e) {
		addNewIdPFields();
	});

	
	function updateJiraIssueCollectorSelector()
	{
		alert("BIN DRIN");
		AJS.$("jiraIssueCollectorSelector").append(AJS.$('<option>', {
			value: 1,
			text: 'TESTCASE'
		}));
		alert("FERTIG");

	}
}); // AJS.toInit

/*
 *  Escape newline chars for being transmitted with JSON over the wire
 */
function escapeNewLineChars(valueToEscape) {
	if (valueToEscape != null && valueToEscape != "") {
		return valueToEscape.replace(/\n/g, "\\n");
	} else {
		return valueToEscape;
	}
}

function replaceNewlinesWithCommas(text) {
	var replaced = text.replace(/[\n\r]/g,',');
	return replaced;
}

function replaceCommasWithNewlines(text) {
	var replaced = text.replace(/,/g,'\n');
	return replaced;
}

function escapeHtmlAndNewlines(text) {
	var entityMap = {
			"&": "&amp;",
			"<": "&lt;",
			">": "&gt;",
			'"': '&quot;',
			"'": '&#39;',
			"/": '&#x2F;'
	};

	if (text != null && text != "") {
		var htmlEscaped = text.replace(/[&<>"'\/]/g, function (s) { return entityMap[s]; });
		return htmlEscaped.replace(/\n/g, "</br>");
	} else {
		return "";
	}
}